# coding: utf-8
from __future__ import unicode_literals

import sys
import logging
from copy import copy
from unittest import TestCase

import poste

sys.path.append(".")

from test_conf import TEST_USERNAME, TEST_PASSWORD, LOG_FILE_POSTE


PREZZO_2LETT_2012 = 0.94  # Euro
PREZZO_2LETT_MULTIDESTINATARIO_2012 = 1.87  # Euro


PREZZO_2LETT_2021 = 1.07  # Euro
PREZZO_2LETT_MULTIDESTINATARIO_2021 = 2.14  # Euro

PREZZO_2LETT_2024 = 1.25  # Euro
PREZZO_2LETT_MULTIDESTINATARIO_2024 = 2.5  # Euro


THUNDER_ADDR = {
    'nome': 'Vittorino',
    'cognome': 'Parenti',
    'ragione_sociale': 'Thunder Systems',
    'citta': 'Pioltello',
    'dug': 'Via',
    'toponimo': 'Dante Alighieri',
    'forza_destinazione': False,
    'cap': '20096',
    'numero_civico': '99',
    'stato': "Italia",
    'provincia': 'MI',
    'tipo_indirizzo': "NORMALE",
}

MITTENTE = {
    'nome': 'Alessandro',
    'cognome': 'Dentella',
    'ragione_sociale': '',
    'citta': 'Usmate Velate',
    'dug': 'Cascina',
    'forza_destinazione': False,
    'toponimo': 'Vega',
    'cap': '20865',
    'numero_civico': '1',
    'stato': "Italia",
    'provincia': 'MB',
    'tipo_indirizzo': "NORMALE",
}

THUNDER_TEST = {
    'nome': "POSTE",
    'cognome': "ITALIANE",
    'ragione_sociale': " ",
    'citta': "ROMA",
    'dug': 'VIALE',
    'toponimo': 'EUROPA',
    'cap': "00144",
    'numero_civico': '175',

    'esponente': " ",
    'zona': " ",
    'tipo_indirizzo': "NORMALE",
    'stato': "Italia",
    'telefono': "abc",
    'forza_destinazione': False,
    'casella_postale': " ",
    'ufficio_postale': " ",
    'provincia': "RM",
    'complemento_nominativo': "DEST CN",
    'complemento_indirizzo': "DEST CI",
    'frazione': " ",
}


class LOLTestClient(TestCase):
    # cliente = "NPCE Client Test"
    cliente = "Fonchim"

    @classmethod
    def setUpClass(cls):
        poste.debug(False)
        logging.getLogger('poste').setLevel(logging.WARNING)
        cls.xol = poste.LOLClient(username=TEST_USERNAME, password=TEST_PASSWORD, service_level='TEST')
        cls.result = None
        cls.xol_submit = None

    def setUp(self):
        self.result = None
        self.xol_submit = None
        self.addr = copy(THUNDER_ADDR)

    def log(self, test_id, guid, msg=''):
        f = open(LOG_FILE_POSTE, 'a')
        f.write("%10.10s %s %s\n" % (test_id, guid, msg))
        f.close()

    def test_01(self):
        """01 - Verifica destinatario"""
        thunder = poste.Destinatario(**self.addr)
        self.result, id = self.xol.valida_destinatari([thunder, ], test=True)
        self.assertEqual('I', self.result.CEResult.Type)
        self.log('01', id)

    def test_01b(self):
        """01b - Verifica destinatazione con lettera accentata"""

        self.addr['cognome'] = u'Fò'
        thunder = poste.Destinatario(**self.addr)
        self.result, id = self.xol.valida_destinatario(thunder, test=True)
        self.assertEqual('I', self.result.CEResult.Type)

    def test_01c(self):
        """01c - Verifica destinazione con indirizzo completo in toponimo senza dug e numero_civico"""

        self.addr['dug'] = u''
        self.addr['toponimo'] = u'Via Dante Alighieri, 99'
        self.addr['numero_civico'] = u''
        thunder = poste.Destinatario(**self.addr)
        self.result = self.xol.valida_destinatario(thunder)
        self.assertEqual('I', self.result.CEResult.Type)

    def test_01d(self):
        """01d - Verifica destinatazione con oggetto Mittente"""

        # mitt = poste.Mittente(**MITTENTE)
        # TODO, valida destinatario sembra non accettare il mittente, è un problema se non funziona questa validazione?
        mitt = poste.Destinatario(**MITTENTE)
        self.result = self.xol.valida_destinatario(mitt)
        self.assertEqual('I', self.result.CEResult.Type)

    def test_02(self):
        """02 - Verifica casella postale"""

        self.addr['casella_postale'] = '99999'
        self.addr['ufficio_postale'] = 'milano 456'
        self.addr['tipo_indirizzo'] = "CASELLA POSTALE"

        thunder = poste.Destinatario(**self.addr)

        self.result, id = self.xol.valida_destinatario(thunder, test=True)
        self.assertEqual('I', self.result.CEResult.Type)
        self.log('02', id)

    def test_03(self):
        """03 - Indirizzo non valido - CAP farlocco"""

        self.addr['cap'] = '00000'
        try:
            self.xol.valida_destinatario(poste.Destinatario(**self.addr), test=True)
            assert False, "A False CAP should raise a complain..."
        except poste.RemoteRecipientValidationError as e:
            self.log("03", e.guid)
            assert True

    def test_03a(self):
        """03a - Indirizzo non valido - citta inesistente"""

        self.addr['citta'] = 'fantasilandia'
        self.assertRaises(poste.RemoteRecipientValidationError,
                          self.xol.valida_destinatario, poste.Destinatario(**self.addr))

    def test_03b(self):
        """03b - Verifica interna: stato mancante"""

        self.addr['stato'] = None
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_04(self):
        """04 - Verifica interna: cognome troppo lungo"""

        self.addr['cognome'] = 'Thunder' * 10
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_05(self):
        """05 - Verifica interna: indirizzo troppo lungo"""

        self.addr['toponimo'] = 'Dante Alighieri' * 10
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_06(self):
        """06 - Verifica interna: frazione + completamento"""

        self.addr['frazione'] = u'Fantasilandia' * 3
        self.addr['complemento_indirizzo'] = u"Che altro c'è" * 3
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_07(self):
        """07 - Verifica interna: complemento nominativo troppo lungo"""

        self.addr['complemento_nominativo'] = 'a' * 51
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_08(self):
        """08 - Verifica casella postale troppo lunga"""

        self.addr['casella_postale'] = '9' * 45
        self.addr['tipo_indirizzo'] = 'CASELLA POSTALE'
        self.assertRaises(poste.RecipientValidationError, poste.Destinatario, **self.addr)

    def test_09(self):
        """09 - Invio lettera a indirizzo normale"""

        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente,
                                     cliente=self.cliente)
        self.assertEqual('0000', self.xol_submit.result.CEResult.Code)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('09', self.xol_submit.guid)

    def test_10(self):
        """10 - Invio lettera a casella postale"""

        self.addr['casella_postale'] = '99999'
        self.addr['ufficio_postale'] = 'milano 456'
        self.addr['tipo_indirizzo'] = 'CASELLA POSTALE'
        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente,
                                     cliente=self.cliente)
        self.assertEqual('0000', self.xol_submit.result.CEResult.Code)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('10', self.xol_submit.guid)

    def test_11(self):
        """11 - Invio lettera e recupero documento"""

        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente,
                                     cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)

        self.result = self.xol.get_documento(self.xol_submit)
        self.assertEqual('I', self.result.CEResult.Type)
        self.log('11', self.xol_submit.guid)

        # res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        # self.assertEqual(res.StatoIdRichiesta, 'Prezzato')

    def test_12(self):
        """12 - Invio documento pdf > A4"""

        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-letter.pdf',
                                     destinatario, mittente, cliente=self.cliente)
        self.log('12', self.xol_submit.guid)

        # Attenzione, lo stato richiesta non basta
        # time.sleep(10)
        # res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        # self.assertEqual(res.StatoIdRichiesta, 'Non Convertito')

    def test_13(self):
        """13 - Invio lettera e annullamento"""

        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente, cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.result = self.xol.annulla(self.xol_submit)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('13', self.xol_submit.guid)

        # res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        # # A volte da: Preso in carico da DCS
        # self.assertEqual(res.StatoIdRichiesta, 'Annullato')

    def test_14(self):
        """14 - Invio lettera e valorizzazione prezzo ... fino ad aumento 1.07 Euro"""

        destinatario1 = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario1,
                                     mittente, cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)

        self.result = self.xol.valorizza(self.xol_submit)
        self.assertEqual('I', self.result.CEResult.Type)

        ser = self.result.ServizioEnquiryResponse[0]
        # Vero fino cambio prezzo
        self.assertEqual(ser.Totale.ImportoNetto, PREZZO_2LETT_2024)

        res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        self.assertEqual(res.StatoIdRichiesta, 'Prezzato')

        self.result = self.xol.confirm(xol_submit=self.xol_submit, autoconfirm=True)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('14', self.xol_submit.guid)

    def test_14b(self):
        """14b - Invio lettera e confirm - autoconfirm = True"""

        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente,
                                     cliente=self.cliente)
        # res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        # self.assertEqual(res.StatoIdRichiesta, 'Presa in Carico DCS')

        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.result = self.xol.confirm(xol_submit=self.xol_submit, autoconfirm=True)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('14b', self.xol_submit.guid)

        # res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        # self.assertEqual(res.StatoIdRichiesta, 'In Attesa di Delivery')

    def test_14c(self):
        """14c - Invio lettera e confirm - autoconfirm = False"""

        self.xol = poste.LOLClient(username=TEST_USERNAME, password=TEST_PASSWORD, service_level='TEST')
        destinatario = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', destinatario, mittente,
                                     cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.result = self.xol.confirm(xol_submit=self.xol_submit, autoconfirm=False)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('14c', self.xol_submit.guid)

    def test_15(self):
        """15 - Invio lettera casella postale, valorizzazione, confirm"""

        self.addr['casella_postale'] = '99999'
        self.addr['ufficio_postale'] = 'milano 456'
        self.addr['tipo_indirizzo'] = 'CASELLA POSTALE'
        destinatario1 = poste.Destinatario(**self.addr)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', [destinatario1, ],
                                     mittente, cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)

        self.result = self.xol.valorizza(self.xol_submit)
        self.assertEqual('I', self.result.CEResult.Type)

        ser = self.result.ServizioEnquiryResponse[0]
        # Vero fino cambio prezzo
        self.assertEqual(ser.Totale.ImportoNetto, PREZZO_2LETT_2024)

        res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        self.assertEqual(res.StatoIdRichiesta, 'Prezzato')

        self.result = self.xol.confirm(xol_submit=self.xol_submit, autoconfirm=True)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('15', self.xol_submit.guid)

    def test_16(self):
        """16 - Invio lettera multidestinatari, valorizzazione, confirm"""

        destinatario1 = poste.Destinatario(**self.addr)
        destinatario2 = poste.Destinatario(**MITTENTE)
        mittente = poste.Mittente(**MITTENTE)
        self.xol_submit = self.xol.submit('pagina-a4.pdf', [destinatario1, destinatario2],
                                     mittente, cliente=self.cliente)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)

        self.result = self.xol.valorizza(self.xol_submit)
        self.assertEqual('I', self.result.CEResult.Type)

        ser = self.result.ServizioEnquiryResponse[0]

        self.assertEqual(ser.Totale.ImportoNetto, PREZZO_2LETT_MULTIDESTINATARIO_2024)

        res = self.xol.get_stato_richiesta(guid=self.xol_submit.guid)
        self.assertEqual(res.StatoIdRichiesta, 'Prezzato')

        self.result = self.xol.confirm(xol_submit=self.xol_submit, autoconfirm=True)
        self.assertEqual('I', self.xol_submit.result.CEResult.Type)
        self.log('16', self.xol_submit.guid)
